"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _default;

var _path2 = _interopRequireDefault(require("path"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _default() {
  let lastOpts;
  let lastCwd;
  let lastPatterns;

  function selectPatterns(state) {
    const {
      opts
    } = state;
    const cwd = state.cwd || process.cwd();
    if (opts === lastOpts && cwd === lastCwd) return lastPatterns;
    lastCwd = cwd;
    lastOpts = opts;
    const {
      packages = [],
      patterns = [],
      paths = []
    } = opts;
    return lastPatterns = [...packages.map(pkg => ({
      pattern: new RegExp(`^${pkg}(/|$)`),
      type: 'package',
      what: pkg
    })), ...patterns.map(_pattern => {
      const pattern = Array.isArray(_pattern) ? new RegExp(_pattern[0], _pattern[1] || '') : new RegExp(_pattern, '');
      return {
        pattern,
        type: 'pattern',
        what: pattern
      };
    }), ...paths.map(_path => ({
      pattern: new RegExp(`^${_path2.default.resolve(cwd, _path)}(/|$)`),
      type: 'path',
      what: _path
    }))];
  }

  return {
    visitor: {
      ImportDeclaration(_path, state) {
        const filename = state.file && state.file.opts && state.file.opts.filename || state.filename;
        const cwd = state.cwd || process.cwd();
        const rawSource = _path.node.source.value;
        const source = /^[./]/.test(rawSource) ? _path2.default.resolve(_path2.default.dirname(filename), rawSource) : rawSource;
        const patterns = selectPatterns(state);
        patterns.forEach(({
          pattern,
          type,
          what
        }) => {
          if (pattern.test(source)) {
            let rule;

            switch (type) {
              case 'package':
                rule = `'${what}'`;
                break;

              case 'pattern':
                rule = String(what);
                break;

              case 'path':
                rule = _path2.default.relative(_path2.default.dirname(filename), _path2.default.resolve(cwd, what));
                if (!/^\./.test(rule)) rule = `./${rule}`;
                rule = `'${rule}'`;
                break;
            }

            throw new Error(`importing from ${rule} is forbidden (imported: '${rawSource}')`);
          }
        });
      }

    }
  };
}